/* 
 *   $Id: ad_pfs_iwrite.c,v 1.4 1997/09/28 17:28:23 thakur Exp $    
 *
 *   Copyright (C) 1997 University of Chicago. 
 *   See COPYRIGHT notice in top-level directory.
 */

#include "ad_pfs.h"
#include "adio_extern.h"

void ADIOI_PFS_IwriteContig(ADIO_File fd, void *buf, int len, int file_ptr_type,
                ADIO_Offset offset, ADIO_Request *request, int *error_code)  
{
    long *id_sys;
    ADIO_Offset off;
    int err;

    *request = ADIOI_Malloc_request();
    (*request)->optype = ADIOI_WRITE;
    (*request)->fd = fd;
    (*request)->next = ADIO_REQUEST_NULL;

    id_sys = (long *) ADIOI_Malloc(sizeof(long));
    (*request)->handle = (void *) id_sys;

    off = (file_ptr_type == ADIO_INDIVIDUAL) ? fd->fp_ind : offset;

    lseek(fd->fd_sys, off, SEEK_SET);
    *id_sys = _iwrite(fd->fd_sys, buf, len);

    if ((*id_sys == -1) && (errno == EQNOMID)) {
     /* the man pages say EMREQUEST, but in reality errno is set to EQNOMID! */

        /* exceeded the max. no. of outstanding requests. */

        /* complete all previous async. requests */
        ADIOI_Complete_async(&err);

        /* try again */
	*id_sys = _iwrite(fd->fd_sys, buf, len);

        if ((*id_sys == -1) && (errno == EQNOMID)) {
            printf("Error in asynchronous I/O\n");
            MPI_Abort(MPI_COMM_WORLD, 1);
        }
    }

    if (file_ptr_type == ADIO_INDIVIDUAL) fd->fp_ind += len; 

    (*request)->queued = 1;
    (*request)->handle_extra = NULL;
    ADIOI_Add_req_to_list(request);
    fd->async_count++;

    fd->fp_sys_posn = -1;   /* set it to null. */

    *error_code = (*id_sys == -1) ? MPI_ERR_UNKNOWN : MPI_SUCCESS;

/* status info. must be linked to the request structure, so that it
   can be accessed later from a wait */
}



void ADIOI_PFS_IwriteStrided(ADIO_File fd, void *buf, int count, 
		       MPI_Datatype datatype, int file_ptr_type,
                       ADIO_Offset offset, ADIO_Request *request, int
                       *error_code)
{

/* offset is in units of etype relative to the filetype. */

    ADIOI_Flatlist_node *flat_buf, *flat_file;
    struct iovec *iov;
    long *id_sys;
    int i, j, k, err, bwr_size, fwr_size=0, st_index=0;
    int bufsize, num, size, sum, n_etypes_in_filetype, size_in_filetype;
    int n_filetypes, etype_in_filetype, abs_off_in_filetype=0;
    int filetype_size, etype_size, buftype_size;
    MPI_Aint filetype_extent, buftype_extent;
    int buf_count, buftype_is_contig, filetype_is_contig, flag;
    char *addr;
    ADIO_Offset off, off_vec, disp;
    ADIO_Request *req;
    int new_bwr_size, new_fwr_size;

/* PFS file pointer modes are not relevant here, because PFS does
   not support strided accesses. */

    if ((fd->iomode != M_ASYNC) && (fd->iomode != M_UNIX)) {
	printf("ADIOI_PFS_IwriteStrided: only M_ASYNC and M_UNIX iomodes are valid\n");
	MPI_Abort(MPI_COMM_WORLD, 1);
    }

    req = request;

    ADIOI_Datatype_iscontig(datatype, &buftype_is_contig);
    ADIOI_Datatype_iscontig(fd->filetype, &filetype_is_contig);

    MPI_Type_size(fd->filetype, &filetype_size);
    MPI_Type_extent(fd->filetype, &filetype_extent);
    MPI_Type_size(datatype, &buftype_size);
    MPI_Type_extent(datatype, &buftype_extent);
    etype_size = fd->etype_size;
    
    bufsize = buftype_size * count;

    if (!buftype_is_contig && filetype_is_contig) {

/* noncontiguous in memory, contiguous in file. */

        ADIOI_Flatten_datatype(datatype);
        flat_buf = ADIOI_Flatlist;
        while (flat_buf->type != datatype) flat_buf = flat_buf->next;

/* There is a limit of 16 on the number of iovecs for ireadv/iwritev! */

        off_vec = (file_ptr_type == ADIO_INDIVIDUAL) ? fd->fp_ind : 
                   fd->disp + etype_size * offset;
	off = off_vec;

        k = 0;
        for (j=0; j<count; j++) 
            for (i=0; i<flat_buf->count; i++) {
		if (!k) iov = (struct iovec *) ADIOI_Malloc(16*sizeof(struct iovec));
                iov[k].iov_base = ((char *) buf) + j*buftype_extent +
                    flat_buf->indices[i]; 
                iov[k].iov_len = flat_buf->blocklens[i];
                off += flat_buf->blocklens[i];
                k = (k+1)%16;

                if (!k) {
		    *req = ADIOI_Malloc_request();
		    (*req)->optype = ADIOI_WRITE;
		    (*req)->fd = fd;
		    (*req)->next = ADIO_REQUEST_NULL;

		    id_sys = (long *) ADIOI_Malloc(sizeof(long));
		    (*req)->handle = (void *) id_sys;

		    lseek(fd->fd_sys, off_vec, SEEK_SET);
		    *id_sys = _iwritev(fd->fd_sys, iov, 16);
/* for some unknown reason, the compiler gives a warning here */

		    if ((*id_sys == -1) && (errno == EQNOMID)) {
     /* the man pages say EMREQUEST, but in reality errno is set to EQNOMID! */

			/* exceeded the max. no. of outstanding requests. */

			/* complete all previous async. requests */
			ADIOI_Complete_async(&err);

			/* try again */
			*id_sys = _iwritev(fd->fd_sys, iov, 16);
/* for some unknown reason, the compiler gives a warning here */

			if ((*id_sys == -1) && (errno == EQNOMID)) {
			    printf("Error in asynchronous I/O\n");
			    MPI_Abort(MPI_COMM_WORLD, 1);
			}
		    }

		    (*req)->queued = 1;
                    (*req)->handle_extra = (void *) iov;
		    ADIOI_Add_req_to_list(req);
		    fd->async_count++;
		    req = &((*req)->next);
		    off_vec = off;
		}
            }

        if (k) {
	    *req = ADIOI_Malloc_request();
	    (*req)->optype = ADIOI_WRITE;
	    (*req)->fd = fd;
	    (*req)->next = ADIO_REQUEST_NULL;
	    
	    id_sys = (long *) ADIOI_Malloc(sizeof(long));
	    (*req)->handle = (void *) id_sys;
	    lseek(fd->fd_sys, off_vec, SEEK_SET);
	    *id_sys = _iwritev(fd->fd_sys, iov, k);
/* for some unknown reason, the compiler gives a warning here */

	    if ((*id_sys == -1) && (errno == EQNOMID)) {
     /* the man pages say EMREQUEST, but in reality errno is set to EQNOMID! */

		/* exceeded the max. no. of outstanding requests. */
		
		/* complete all previous async. requests */
		ADIOI_Complete_async(&err);
		
		/* try again */
		*id_sys = _iwritev(fd->fd_sys, iov, k);
/* for some unknown reason, the compiler gives a warning here */
		
		if ((*id_sys == -1) && (errno == EQNOMID)) {
		    printf("Error in asynchronous I/O\n");
		    MPI_Abort(MPI_COMM_WORLD, 1);
		}
	    }

	    (*req)->queued = 1;
            (*req)->handle_extra = (void *) iov;
	    ADIOI_Add_req_to_list(req);
	    fd->async_count++;
	    req = &((*req)->next);
	}

        if (file_ptr_type == ADIO_INDIVIDUAL) fd->fp_ind = off;

        *error_code = (*id_sys == -1) ? MPI_ERR_UNKNOWN : MPI_SUCCESS;
    }

    else {  /* noncontiguous in file */

/* split up into several contiguous writes */

/* find starting location in the file */

/* filetype already flattened in ADIO_Open */

	flat_file = ADIOI_Flatlist;
	while (flat_file->type != fd->filetype) flat_file = flat_file->next;
        disp = fd->disp;

	if (file_ptr_type == ADIO_INDIVIDUAL) {
	    offset = fd->fp_ind; /* in bytes */
            n_filetypes = -1;
            flag = 0;
            while (!flag) {
                n_filetypes++;
                for (i=0; i<flat_file->count; i++) {
                    if (disp + flat_file->indices[i] + 
                        n_filetypes*filetype_extent + flat_file->blocklens[i] 
                            >= offset) {
                        st_index = i;
                        fwr_size = disp + flat_file->indices[i] + 
                                n_filetypes*filetype_extent
                                 + flat_file->blocklens[i] - offset;
                        flag = 1;
                        break;
                    }
                }
            }
	}
	else {
	    n_etypes_in_filetype = filetype_size/etype_size;
	    n_filetypes = offset / n_etypes_in_filetype;
	    etype_in_filetype = offset % n_etypes_in_filetype;
	    size_in_filetype = etype_in_filetype * etype_size;
 
	    sum = 0;
	    for (i=0; i<flat_file->count; i++) {
		sum += flat_file->blocklens[i];
		if (sum > size_in_filetype) {
		    st_index = i;
		    fwr_size = sum - size_in_filetype;
		    abs_off_in_filetype = flat_file->indices[i] +
			size_in_filetype - (sum - flat_file->blocklens[i]);
		    break;
		}
	    }

	    /* abs. offset in bytes in the file */
            offset = disp + n_filetypes*filetype_extent + abs_off_in_filetype;
	}

	if (buftype_is_contig && !filetype_is_contig) {

/* contiguous in memory, noncontiguous in file. should be the most
   common case. */

	    i = 0;
	    j = st_index;
	    off = offset;
	    fwr_size = ADIOI_MIN(fwr_size, bufsize);
	    while (i < bufsize) {
                if (fwr_size) { 
                    /* TYPE_UB and TYPE_LB can result in 
                       fwr_size = 0. save system call in such cases */ 

		    /* lseek(fd->fd_sys, off, SEEK_SET);
		    err = write(fd->fd_sys, ((char *) buf) + i, fwr_size); */

		    ADIOI_PFS_IwriteContig(fd, ((char *) buf) + i, fwr_size,
 			      ADIO_EXPLICIT_OFFSET, off, req, &err);
		    req = &((*req)->next);
		}
		i += fwr_size;

                if (off + fwr_size < disp + flat_file->indices[j] +
                   flat_file->blocklens[j] + n_filetypes*filetype_extent)
                       off += fwr_size;
                /* did not reach end of contiguous block in filetype.
                   no more I/O needed. off is incremented by fwr_size. */
                else {
		    if (j < (flat_file->count - 1)) j++;
		    else {
			j = 0;
			n_filetypes++;
		    }
		    off = disp + flat_file->indices[j] + 
			             n_filetypes*filetype_extent;
		    fwr_size = ADIOI_MIN(flat_file->blocklens[j], bufsize-i);
		}
	    }
	}
	else {
/* noncontiguous in memory as well as in file */

	    ADIOI_Flatten_datatype(datatype);
	    flat_buf = ADIOI_Flatlist;
	    while (flat_buf->type != datatype) flat_buf = flat_buf->next;

	    k = num = buf_count = 0;
	    i = flat_buf->indices[0];
	    j = st_index;
	    off = offset;
	    bwr_size = flat_buf->blocklens[0];

	    while (num < bufsize) {
		size = ADIOI_MIN(fwr_size, bwr_size);
		if (size) {
/*		    lseek(fd->fd_sys, off, SEEK_SET);
		    err = write(fd->fd_sys, ((char *) buf) + i, size);*/
		    ADIOI_PFS_IwriteContig(fd, ((char *) buf) + i, size, 
                           ADIO_EXPLICIT_OFFSET, off, req, &err);
		    req = &((*req)->next);
		}

		new_fwr_size = fwr_size;
		new_bwr_size = bwr_size;

		if (size == fwr_size) {
/* reached end of contiguous block in file */
                    if (j < (flat_file->count - 1)) j++;
                    else {
                        j = 0;
                        n_filetypes++;
                    }

                    off = disp + flat_file->indices[j] + 
                                              n_filetypes*filetype_extent;

		    new_fwr_size = flat_file->blocklens[j];
		    if (size != bwr_size) {
			i += size;
			new_bwr_size -= size;
		    }
		}

		if (size == bwr_size) {
/* reached end of contiguous block in memory */

		    k = (k + 1)%flat_buf->count;
		    buf_count++;
		    i = buftype_extent*(buf_count/flat_buf->count) +
			flat_buf->indices[k]; 
		    new_bwr_size = flat_buf->blocklens[k];
		    if (size != fwr_size) {
			off += size;
			new_fwr_size -= size;
		    }
		}
		num += size;
		fwr_size = new_fwr_size;
                bwr_size = new_bwr_size;
	    }
	}

        if (file_ptr_type == ADIO_INDIVIDUAL) fd->fp_ind = off;
	/* check if file pointer is updated correctly to point the start 
          of the next etype. I think it is. */

	*error_code = (err == -1) ? MPI_ERR_UNKNOWN : MPI_SUCCESS;
    }

    fd->fp_sys_posn = -1;   /* set it to null. */

    if (!buftype_is_contig) ADIOI_Delete_flattened(datatype);
}
