/* 
 *   $Id: ad_piofs_wrcoll.c,v 1.2 1997/09/04 20:44:13 thakur Exp $    
 *
 *   Copyright (C) 1997 University of Chicago. 
 *   See COPYRIGHT notice in top-level directory.
 */

#include "ad_piofs.h"
#include "adio_extern.h"

void ADIOI_PIOFS_WriteStridedColl(ADIO_File fd, void *buf, int count,
                       MPI_Datatype datatype, int file_ptr_type,
                       ADIO_Offset offset, ADIO_Status *status, int
                       *error_code)
{
    int filetype_is_contig, buftype_is_contig, size, bufsize;
    ADIO_Offset off;

/* since PIOFS does not support file locking, the collective write
   algorithm can only be used if the file has been opened with
   ADIO_UNIQUE_OPEN. */
  
    if (fd->access_mode & ADIO_UNIQUE_OPEN)
	ADIOI_GEN_WriteStridedColl(fd, buf, count, datatype, file_ptr_type,
				   offset, status, error_code);
    else {
	ADIOI_Datatype_iscontig(datatype, &buftype_is_contig);
	ADIOI_Datatype_iscontig(fd->filetype, &filetype_is_contig);

    /* contiguous or strided? */

	if (buftype_is_contig && filetype_is_contig) {

	    MPI_Type_size(datatype, &size);
	    bufsize = size * count;
	    
	    if (file_ptr_type == ADIO_INDIVIDUAL)
		ADIO_WriteContig(fd, buf, bufsize, ADIO_INDIVIDUAL,
                     0, status, error_code); 
	    else {
		size = fd->etype_size;
		off = fd->disp + size * offset;
		ADIO_WriteContig(fd, buf, bufsize, ADIO_EXPLICIT_OFFSET,
				 off, status, error_code);
	    }
	}
	else ADIO_WriteStrided(fd, buf, count, datatype, file_ptr_type,
				   offset, status, error_code);
    }
}
