/* 
 *   $Id: dtype.c,v 1.3 1997/09/28 17:29:29 thakur Exp $    
 *
 *   Copyright (C) 1997 University of Chicago. 
 *   See COPYRIGHT notice in top-level directory.
 */

#ifdef __MPICH

/* MPI-2 datatype accessor functions for MPICH */

#define ANSI_ARGS(a) a

#include "adio.h"
#include "cookie.h"
#include "datatype.h"
#include "mpich2.h"

int MPI_Type_get_envelope(MPI_Datatype datatype, int *num_integers,
		  int *num_addresses, int *num_datatypes, int *combiner) 
{
    struct MPIR_DATATYPE *dtypeptr;

    dtypeptr = MPIR_GET_DTYPE_PTR(datatype);

    switch (dtypeptr->dte_type) {
    case MPIR_CONTIG:
	*num_integers = 1;
	*num_addresses = 0;
	*num_datatypes = 1;
	*combiner = MPI_COMBINER_CONTIGUOUS;
	break;
    case MPIR_VECTOR:
	*combiner = MPI_COMBINER_VECTOR;
	*num_integers = 3;
	*num_addresses = 0;
	*num_datatypes = 1;
	break;
    case MPIR_HVECTOR:
	*combiner = MPI_COMBINER_HVECTOR;
	*num_integers = 2;
	*num_addresses = 1;
	*num_datatypes = 1;
	break;
    case MPIR_INDEXED:
	*combiner = MPI_COMBINER_INDEXED;
	*num_integers = 1 + 2*dtypeptr->count;
	*num_addresses = 0;
	*num_datatypes = 1;
	break;
    case MPIR_HINDEXED:
	*combiner = MPI_COMBINER_HINDEXED;
	*num_integers = 1 + dtypeptr->count;
	*num_addresses = dtypeptr->count;
	*num_datatypes = 1;
	break;
    case MPIR_STRUCT:
	*combiner = MPI_COMBINER_STRUCT;
	*num_integers = 1 + dtypeptr->count;
	*num_addresses = dtypeptr->count;
	*num_datatypes = dtypeptr->count;
	break;
    default:  
	*combiner = MPI_COMBINER_NAMED;
	*num_integers = 0;
	*num_addresses = 0;
	*num_datatypes = 0;
	break;
    }

    return MPI_SUCCESS;
}



int MPI_Type_get_contents(MPI_Datatype datatype, int max_integers, int
              max_addresses, int max_datatypes, int *array_of_integers, 
	      MPI_Aint *array_of_addresses, MPI_Datatype *array_of_datatypes) 
{
    int i;
    struct MPIR_DATATYPE *dtypeptr;

    dtypeptr = MPIR_GET_DTYPE_PTR(datatype);

    switch (dtypeptr->dte_type) {
    case MPIR_CONTIG:
	array_of_integers[0] = dtypeptr->count;
	array_of_datatypes[0] = dtypeptr->old_type->self;
	break;
    case MPIR_VECTOR:  /* In MPICH, vector is internally represented
			  as hvector. */
    case MPIR_HVECTOR:
	array_of_integers[0] = dtypeptr->count;
	array_of_integers[1] = dtypeptr->blocklen;
	array_of_addresses[0] = dtypeptr->stride;
	array_of_datatypes[0] = dtypeptr->old_type->self;
	break;
    case MPIR_INDEXED: /* In MPICH, indexed is internally represented
			  as hindexed. */
    case MPIR_HINDEXED:
	array_of_integers[0] = dtypeptr->count;
	for (i=1; i<=dtypeptr->count; i++)
	    array_of_integers[i] = dtypeptr->blocklens[i-1];
	for (i=0; i<dtypeptr->count; i++)
	    array_of_addresses[i] = dtypeptr->indices[i];
	array_of_datatypes[0] = dtypeptr->old_type->self;
	break;
    case MPIR_STRUCT:
	array_of_integers[0] = dtypeptr->count;
	for (i=1; i<=dtypeptr->count; i++)
	    array_of_integers[i] = dtypeptr->blocklens[i-1];
	for (i=0; i<dtypeptr->count; i++)
	    array_of_addresses[i] = dtypeptr->indices[i];
	for (i=0; i<dtypeptr->count; i++)
	    array_of_datatypes[i] = dtypeptr->old_types[i]->self;
	break;
    default:  
	printf("Error: basic datatype passed to MPI_Type_get_contents\n");
	MPI_Abort(MPI_COMM_WORLD, 1);
    }
    return MPI_SUCCESS;
}

#endif
